@@ -288,6 +288,81 @@ done:
     return ret;
 }
 
+int confdb_set_bool(struct confdb_ctx *cdb,
+                     const char *section,
+                     const char *attribute,
+                     bool val)
+{
+    TALLOC_CTX *tmp_ctx;
+    struct ldb_dn *dn;
+    char *secdn;
+    struct ldb_message *msg;
+    int ret, lret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (!tmp_ctx)
+        return ENOMEM;
+
+    ret = parse_section(tmp_ctx, section, &secdn, NULL);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    dn = ldb_dn_new(tmp_ctx, cdb->ldb, secdn);
+    if (!dn) {
+        ret = EIO;
+        goto done;
+    }
+
+    msg = ldb_msg_new(tmp_ctx);
+    if (!msg) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    msg->dn = dn;
+
+    lret = ldb_msg_add_empty(msg, attribute, LDB_FLAG_MOD_REPLACE, NULL);
+    if (lret != LDB_SUCCESS) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("ldb_msg_add_empty failed: [%s]\n", ldb_strerror(lret)));
+        ret = EIO;
+        goto done;
+    }
+
+    if (val) {
+        lret = ldb_msg_add_string(msg, attribute, "True");
+    } else {
+        lret = ldb_msg_add_string(msg, attribute, "False");
+    }
+    if (lret != LDB_SUCCESS) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("ldb_msg_add_string failed: [%s]\n", ldb_strerror(lret)));
+        ret = EIO;
+        goto done;
+    }
+
+
+    lret = ldb_modify(cdb->ldb, msg);
+    if (lret != LDB_SUCCESS) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("ldb_modify failed: [%s]\n", ldb_strerror(lret)));
+        ret = EIO;
+        goto done;
+    }
+
+    ret = EOK;
+
+done:
+    talloc_free(tmp_ctx);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("Failed to set [%s] from [%s], error [%d] (%s)\n",
+               attribute, section, ret, strerror(ret)));
+    }
+    return ret;
+}
+
 int confdb_get_string(struct confdb_ctx *cdb, TALLOC_CTX *ctx,
                       const char *section, const char *attribute,
                       const char *defstr, char **result)
@@ -337,15 +412,22 @@ failed:
     return ret;
 }
 
-int confdb_get_int(struct confdb_ctx *cdb, TALLOC_CTX *ctx,
+int confdb_get_int(struct confdb_ctx *cdb,
                    const char *section, const char *attribute,
                    int defval, int *result)
 {
     char **values = NULL;
     long val;
     int ret;
+    TALLOC_CTX *tmp_ctx;
 
-    ret = confdb_get_param(cdb, ctx, section, attribute, &values);
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        ret = ENOMEM;
+        goto failed;
+    }
+
+    ret = confdb_get_param(cdb, tmp_ctx, section, attribute, &values);
     if (ret != EOK) {
         goto failed;
     }
@@ -373,27 +455,34 @@ int confdb_get_int(struct confdb_ctx *cdb, TALLOC_CTX *ctx,
         val = defval;
     }
 
-    talloc_free(values);
+    talloc_free(tmp_ctx);
 
     *result = (int)val;
     return EOK;
 
 failed:
-    talloc_free(values);
+    talloc_free(tmp_ctx);
     DEBUG(1, ("Failed to read [%s] from [%s], error [%d] (%s)\n",
               attribute, section, ret, strerror(ret)));
     return ret;
 }
 
-long confdb_get_long(struct confdb_ctx *cdb, TALLOC_CTX *ctx,
+long confdb_get_long(struct confdb_ctx *cdb,
                      const char *section, const char *attribute,
                      long defval, long *result)
 {
     char **values = NULL;
     long val;
     int ret;
+    TALLOC_CTX *tmp_ctx;
 
-    ret = confdb_get_param(cdb, ctx, section, attribute, &values);
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        ret = ENOMEM;
+        goto failed;
+    }
+
+    ret = confdb_get_param(cdb, tmp_ctx, section, attribute, &values);
     if (ret != EOK) {
         goto failed;
     }
@@ -416,27 +505,34 @@ long confdb_get_long(struct confdb_ctx *cdb, TALLOC_CTX *ctx,
         val = defval;
     }
 
-    talloc_free(values);
+    talloc_free(tmp_ctx);
 
     *result = val;
     return EOK;
 
 failed:
-    talloc_free(values);
+    talloc_free(tmp_ctx);
     DEBUG(1, ("Failed to read [%s] from [%s], error [%d] (%s)\n",
               attribute, section, ret, strerror(ret)));
     return ret;
 }
 
-int confdb_get_bool(struct confdb_ctx *cdb, TALLOC_CTX *ctx,
+int confdb_get_bool(struct confdb_ctx *cdb,
                     const char *section, const char *attribute,
                     bool defval, bool *result)
 {
     char **values = NULL;
     bool val;
     int ret;
+    TALLOC_CTX *tmp_ctx;
 
-    ret = confdb_get_param(cdb, ctx, section, attribute, &values);
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        ret = ENOMEM;
+        goto failed;
+    }
+
+    ret = confdb_get_param(cdb, tmp_ctx, section, attribute, &values);
     if (ret != EOK) {
         goto failed;
     }
@@ -465,13 +561,13 @@ int confdb_get_bool(struct confdb_ctx *cdb, TALLOC_CTX *ctx,
         val = defval;
     }
 
-    talloc_free(values);
+    talloc_free(tmp_ctx);
 
     *result = val;
     return EOK;
 
 failed:
-    talloc_free(values);
+    talloc_free(tmp_ctx);
     DEBUG(1, ("Failed to read [%s] from [%s], error [%d] (%s)\n",
               attribute, section, ret, strerror(ret)));
     return ret;
@@ -706,6 +802,7 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         ret = ENOMEM;
         goto done;
     }
+    domain->conn_name = domain->name;
 
     tmp = ldb_msg_find_attr_as_string(res->msgs[0],
                                       CONFDB_DOMAIN_ID_PROVIDER,
@@ -748,7 +845,7 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         tmp = ldb_msg_find_attr_as_string(res->msgs[0],
                                           CONFDB_DOMAIN_ACCESS_PROVIDER,
                                           NULL);
-        if (tmp && strcasecmp(tmp, "local") != 0) {
+        if (tmp && strcasecmp(tmp, "permit") != 0) {
             DEBUG(0, ("Local ID provider does not support [%s] as an ACCESS provider.\n", tmp));
             ret = EINVAL;
             goto done;
@@ -786,7 +883,7 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         }
     }
     if (!domain->enumerate) {
-        DEBUG(1, ("No enumeration for [%s]!\n", domain->name));
+        DEBUG(SSSDBG_TRACE_FUNC, ("No enumeration for [%s]!\n", domain->name));
     }
 
     /* Determine if user/group names will be Fully Qualified
@@ -889,7 +986,7 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         goto done;
     }
 
-    /* Override the service cache timeout, if specified */
+    /* Override the autofs cache timeout, if specified */
     ret = get_entry_as_uint32(res->msgs[0], &domain->autofsmap_timeout,
                               CONFDB_DOMAIN_AUTOFS_CACHE_TIMEOUT,
                               entry_cache_timeout);
@@ -900,6 +997,35 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
         goto done;
     }
 
+    /* Override the sudo cache timeout, if specified */
+    ret = get_entry_as_uint32(res->msgs[0], &domain->sudo_timeout,
+                              CONFDB_DOMAIN_SUDO_CACHE_TIMEOUT,
+                              entry_cache_timeout);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              ("Invalid value for [%s]\n",
+               CONFDB_DOMAIN_SUDO_CACHE_TIMEOUT));
+        goto done;
+    }
+
+    /* Set the PAM warning time, if specified */
+    val = ldb_msg_find_attr_as_int(res->msgs[0],
+                                   CONFDB_DOMAIN_PWD_EXPIRATION_WARNING,
+                                   -1);
+    if (val > 0) {
+        /* The value is in days, transform it to seconds */
+        val *= 24 * 3600;
+    } else {
+        ret = confdb_get_int(cdb, CONFDB_PAM_CONF_ENTRY,
+                             CONFDB_PAM_PWD_EXPIRATION_WARNING,
+                             -1, &val);
+        if (ret != EOK) {
+            DEBUG(1, ("Failed to read PAM expiration warning, not fatal.\n"));
+            val = -1;
+        }
+    }
+    domain->pwd_expiration_warning = val;
+
     ret = get_entry_as_uint32(res->msgs[0], &domain->override_gid,
                               CONFDB_DOMAIN_OVERRIDE_GID, 0);
     if (ret != EOK) {
@@ -909,10 +1035,53 @@ static int confdb_get_domain_internal(struct confdb_ctx *cdb,
 
     tmp = ldb_msg_find_attr_as_string(res->msgs[0],
                                       CONFDB_NSS_OVERRIDE_HOMEDIR, NULL);
-    domain->override_homedir = talloc_strdup(domain, tmp);
-    if (!domain->name) {
-        ret = ENOMEM;
-        goto done;
+    if (tmp != NULL) {
+        domain->override_homedir = talloc_strdup(domain, tmp);
+        if (!domain->override_homedir) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    tmp = ldb_msg_find_attr_as_string(res->msgs[0],
+                                      CONFDB_NSS_FALLBACK_HOMEDIR, NULL);
+    if (tmp != NULL) {
+        domain->fallback_homedir = talloc_strdup(domain, tmp);
+        if (!domain->fallback_homedir) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    tmp = ldb_msg_find_attr_as_string(res->msgs[0],
+                                      CONFDB_DOMAIN_SUBDOMAIN_HOMEDIR,
+                                      CONFDB_DOMAIN_DEFAULT_SUBDOMAIN_HOMEDIR);
+    if (tmp != NULL) {
+        domain->subdomain_homedir = talloc_strdup(domain, tmp);
+        if (!domain->subdomain_homedir) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    tmp = ldb_msg_find_attr_as_string(res->msgs[0],
+                                      CONFDB_NSS_OVERRIDE_SHELL, NULL);
+    if (tmp != NULL) {
+        domain->override_shell = talloc_strdup(domain, tmp);
+        if (!domain->override_shell) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    tmp = ldb_msg_find_attr_as_string(res->msgs[0],
+                                      CONFDB_NSS_DEFAULT_SHELL, NULL);
+    if (tmp != NULL) {
+        domain->default_shell = talloc_strdup(domain, tmp);
+        if (!domain->default_shell) {
+            ret = ENOMEM;
+            goto done;
+        }
     }
 
     ret = get_entry_as_bool(res->msgs[0], &domain->case_sensitive,
