@@ -31,6 +31,7 @@
 #include "db/sysdb_sudo.h"
 
 struct sdap_sudo_full_refresh_state {
+    struct sdap_sudo_ctx *sudo_ctx;
     struct sdap_id_ctx *id_ctx;
     struct sysdb_ctx *sysdb;
     int dp_error;
@@ -128,6 +129,10 @@ int sdap_sudo_init(struct be_ctx *be_ctx,
     *ops = &sdap_sudo_ops;
     *pvt_data = sudo_ctx;
 
+    /* we didn't do any full refresh now,
+     * so we don't have current usn values available */
+    sudo_ctx->full_refresh_done = false;
+
     ret = ldap_get_sudo_options(id_ctx, be_ctx->cdb,
                                 be_ctx->conf_path, id_ctx->opts,
                                 &sudo_ctx->use_host_filter,
@@ -322,6 +327,13 @@ static char *sdap_sudo_build_host_filter(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
+    /* sudoHost is not specified */
+    filter = talloc_asprintf_append_buffer(filter, "(!(%s=*))",
+                                           map[SDAP_AT_SUDO_HOST].name);
+    if (filter == NULL) {
+        goto done;
+    }
+
     /* ALL */
     filter = talloc_asprintf_append_buffer(filter, "(%s=ALL)",
                                            map[SDAP_AT_SUDO_HOST].name);
@@ -532,6 +544,7 @@ static struct tevent_req *sdap_sudo_full_refresh_send(TALLOC_CTX *mem_ctx,
         return NULL;
     }
 
+    state->sudo_ctx = sudo_ctx;
     state->id_ctx = id_ctx;
     state->sysdb = id_ctx->be->sysdb;
 
@@ -606,6 +619,8 @@ static void sdap_sudo_full_refresh_done(struct tevent_req *subreq)
         return;
     }
 
+    state->sudo_ctx->full_refresh_done = true;
+
     /* save the time in the sysdb */
     ret = sysdb_sudo_set_last_full_refresh(state->sysdb, time(NULL));
     if (ret != EOK) {
@@ -810,6 +825,7 @@ static struct tevent_req *sdap_sudo_smart_refresh_send(TALLOC_CTX *mem_ctx,
     struct sdap_sudo_smart_refresh_state *state = NULL;
     char *ldap_filter = NULL;
     char *ldap_full_filter = NULL;
+    const char *usn;
     int ret;
 
     req = tevent_req_create(mem_ctx, &state, struct sdap_sudo_smart_refresh_state);
@@ -818,9 +834,11 @@ static struct tevent_req *sdap_sudo_smart_refresh_send(TALLOC_CTX *mem_ctx,
         return NULL;
     }
 
-    if (srv_opts == NULL || srv_opts->max_sudo_value == 0) {
-        /* Perform full refresh */
-        DEBUG(SSSDBG_TRACE_FUNC, ("USN value is unknown!\n"));
+    if (!sudo_ctx->full_refresh_done
+            && (srv_opts == NULL || srv_opts->max_sudo_value == 0)) {
+        /* Perform full refresh first */
+        DEBUG(SSSDBG_TRACE_FUNC, ("USN value is unknown, "
+                                  "waiting for full refresh!\n"));
         ret = EINVAL;
         goto immediately;
     }
@@ -829,12 +847,11 @@ static struct tevent_req *sdap_sudo_smart_refresh_send(TALLOC_CTX *mem_ctx,
     state->sysdb = id_ctx->be->sysdb;
 
     /* Download all rules from LDAP that are newer than usn */
+    usn = srv_opts->max_sudo_value == NULL ? "0" : srv_opts->max_sudo_value;
     ldap_filter = talloc_asprintf(state, "(&(objectclass=%s)(%s>=%s)(!(%s=%s)))",
                                   map[SDAP_OC_SUDORULE].name,
-                                  map[SDAP_AT_SUDO_USN].name,
-                                  srv_opts->max_sudo_value,
-                                  map[SDAP_AT_SUDO_USN].name,
-                                  srv_opts->max_sudo_value);
+                                  map[SDAP_AT_SUDO_USN].name, usn,
+                                  map[SDAP_AT_SUDO_USN].name, usn);
     if (ldap_filter == NULL) {
         ret = ENOMEM;
         goto immediately;
@@ -939,10 +956,16 @@ static void sdap_sudo_periodical_first_refresh_done(struct tevent_req *req)
     }
 
     ret = sdap_sudo_full_refresh_recv(subreq, &dp_error, &error);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Periodical full refresh of sudo rules "
+                                  "failed [%d]: %s)\n", ret, strerror(ret)));
+        goto schedule;
+    }
+
     if (dp_error != DP_ERR_OK || error != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, ("Periodical full refresh of sudo rules "
-              "failed [dp_error: %d] ([%d]: %s)\n",
-              dp_error, error, strerror(error)));
+                                  "failed [dp_error: %d] ([%d]: %s)\n",
+                                  dp_error, error, strerror(error)));
         goto schedule;
     }
 
@@ -986,8 +1009,8 @@ static void sdap_sudo_periodical_full_refresh_done(struct tevent_req *req)
     struct tevent_req *subreq = NULL; /* req from sdap_sudo_full_refresh_send() */
     struct sdap_sudo_ctx *sudo_ctx = NULL;
     time_t delay;
-    int dp_error = DP_ERR_OK;
-    int error = EOK;
+    int dp_error;
+    int error;
     int ret;
 
     ret = sdap_sudo_timer_recv(req, req, &subreq);
@@ -998,10 +1021,16 @@ static void sdap_sudo_periodical_full_refresh_done(struct tevent_req *req)
     }
 
     ret = sdap_sudo_full_refresh_recv(subreq, &dp_error, &error);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Periodical full refresh of sudo rules "
+                                  "failed [%d]: %s)\n", ret, strerror(ret)));
+        goto schedule;
+    }
+
     if (dp_error != DP_ERR_OK || error != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, ("Periodical full refresh of sudo rules "
-                      "failed [dp_error: %d] ([%d]: %s)\n",
-                      dp_error, error, strerror(error)));
+                                  "failed [dp_error: %d] ([%d]: %s)\n",
+                                  dp_error, error, strerror(error)));
         goto schedule;
     }
 
@@ -1041,10 +1070,15 @@ static void sdap_sudo_periodical_smart_refresh_done(struct tevent_req *req)
     }
 
     ret = sdap_sudo_smart_refresh_recv(subreq, &dp_error, &error);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Periodical smart refresh of sudo rules "
+                                  "failed [%d]: %s\n", ret, strerror(ret)));
+    }
+
     if (dp_error != DP_ERR_OK || error != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, ("Periodical smart refresh of sudo rules "
-              "failed [dp_error: %d] ([%d]: %s)\n",
-              dp_error, error, strerror(error)));
+                                  "failed [dp_error: %d] ([%d]: %s)\n",
+                                  dp_error, error, strerror(error)));
         goto schedule;
     }
 
